from . import data

class Camera:
    def __init__(self):
        self.position = data.camera_position
        self.return_change : bool = False

    @staticmethod
    def set_screen_center(x : int, y : int):
        data.screen_center = {"x" : x, "y" : y}
        data.camera_position["x"] = x
        data.camera_position["y"] = y
    @staticmethod
    def get_screen_center(string : str = 1):
        return int(data.screen_center[string])
    
    @staticmethod
    def set_minimum(minimum : float):
        data.minimum = minimum

    def set_return_change(self, value : bool):
        self.return_change = value

    def move(self, x, y):
        self.position["x"] += x
        self.position["y"] += y
        if self.return_change:
            return self.position

    def set_position(self, x, y):
        self.position["x"] = x
        self.position["y"] = y
        if self.return_change:
            return self.position

    def get_position(self):
        return self.position



    def add_size(self, size):
        if data.camera_size + size <= data.minimum:
            return
        data.camera_size += size
        if self.return_change:
            return data.camera_size

    def set_size(self, size):
        data.camera_size = size
        if self.return_change:
            return data.camera_size

    def get_size(self):
        return data.camera_size




    def turn_left(self, angle):
        data.camera_d += angle
        if self.return_change:
            return data.camera_d

    def turn_right(self, angle):
        data.camera_d -= angle
        if self.return_change:
            return data.camera_d
    
    def set_direction(self, angle):
        data.camera_d = angle
        if self.return_change:
            return data.camera_d

    def get_direction(self):
        return data.camera_d
    

Camera = Camera()

__all__ = ["Camera"]


import pygame
import os
import sys
from . import image_scrolling
import math
import subprocess
from tkinter import messagebox, filedialog, simpledialog
import tkinter as tk
import json
from rgb import RGB
from data import (
    ae_images_position_x, 
    ae_images_position_y, 
    ae_images_path, 
    ae_images_direction, 
    ae_images_size_x, 
    ae_images_size_y, 
    ae_next_id, 
    ae_images_id, 
    ae_images_type, 
    ae_every_frame_position_x, 
    ae_every_frame_position_y, 
    ae_every_frame_path, 
    ae_every_frame_direction, 
    ae_every_frame_size_x, 
    ae_every_frame_size_y, 
    ae_every_frame_type
)

_is_tk = False
_data = {"animation_name":"untitled word", "frame":int(1)}

def _display_button(screen : pygame.Surface, font_path, x, y, w, h, text, color, text_color, font_size=20):
    font = pygame.font.Font(font_path, font_size)
    pygame.draw.rect(
        screen, 
        color, 
        (
           x, y, w, h 
        )
    )
    text_surface = font.render(text, True, text_color)
    screen.blit(text_surface, text_surface.get_rect(center=(x + w // 2, y + h // 2)))
    
def _display_text(screen : pygame.Surface, font_path, x, y, text, color):
    font = pygame.font.Font(font_path, 20)
    text_surface = font.render(text, True, color)
    screen.blit(text_surface, text_surface.get_rect(center=(x, y)))

def _get_value(string, index, default="0"):
    """从分号分隔的字符串中获取第index个值"""
    if not string:
        return default
    parts = string.split(";")
    if index < len(parts):
        return parts[index]
    return default



# 按钮按下后的事件处理函数
def _import_button(grid : image_scrolling.InfiniteImageGrid):
    image_path = filedialog.askopenfilename(
        title="选择图片", 
        filetypes=[("图片文件", "*.jpg *.png *.jpeg")]
    )
    if image_path:
        grid.add_image(image_path)


def _settings_button():
    global _is_tk
    def _set_animation_name():
        name = simpledialog.askstring("设置动画名称", "请输入动画名称：")
        if name:
            _data["animation_name"] = name
        else:
            messagebox.showerror("Error", "请输入动画名称！")


    _is_tk = True
    loop = tk.Tk()
    loop.title("settings")

    icon = tk.PhotoImage(file=os.path.join(os.path.dirname(__file__), "Image/tk_icon.gif"))
    loop.iconphoto(False, icon)

    loop.geometry("600x800")
    loop.resizable(False, False)
    loop.configure(bg=RGB(16, 16, 16).of())

    # Text
    notice_text = tk.Label(loop, 
                           text="⚠所有设置关闭窗口后会生效！", 
                           font=(os.path.join(os.path.dirname(__file__), "font", "MSYHBD.TTC"), 16, "bold"), 
                           bg=RGB(16, 16, 16).of(), 
                           fg=RGB(255, 0, 0).of()
    )
    notice_text.pack()
    

    # Botton
    set_name_button = tk.Button(loop, 
                                text="设置动画名称", 
                                font=(os.path.join(os.path.dirname(__file__), "font", "MSYHBD.TTC"), 16), 
                                command=_set_animation_name, 
                                bg=RGB(50, 50, 50).of(), 
                                fg=RGB(255, 255, 255).of()
    )
    set_name_button.pack()

    loop.mainloop()
    _is_tk = False





def _collisions(x, y, w, h, x_2, y_2, w_2, h_2):
    if x < x_2 + w_2 and x + w > x_2 and y < y_2 + h_2 and y + h > y_2:
        return True
    else:
        return False




class AnimationEngine:
    def __init__(self):
        pygame.init()
        pygame.font.init()

        self.YELLOW = (255, 255, 0)

        self.dirpath = os.path.dirname(__file__)
        self.font_path = os.path.join(self.dirpath, "font", "MSYHBD.TTC")
        self.font = pygame.font.Font(self.font_path)
        self.FPS = 60
        self.animation_engine_window = pygame.display.set_mode((1600, 850), pygame.DOUBLEBUF)
        self.editor_images_cache = {}  # 缓存编辑器中的图片
        self.is_has_image_drag = {}  # 是否有图片被拖动dict
        self.is_has_image_display_attribution = {}  # 是否有图片显示属性dict
        self.is_rotateing = {}  # 是否正在旋转dict
        self.is_sizing = {}  # 是否正在缩放dict
        self.current_dragging_key = None  # 当前正在拖动的图片key
        self.is_dragging = False  # 是否正在拖动中
        self._dirty_caches = set()

        self.end_x = 0
        self.end_y = 0



        # import button
        self.import_button_rect_x, self.import_button_rect_y = 5, 3
        self.import_button_rect_width, self.import_button_rect_height = 80, 30
        self.import_button_text = "导入图片"
        self.import_button_color, self.import_button_text_color = (50, 50, 50), (255, 255, 255)

        # settings button
        self.settings_button_rect_x, self.settings_button_rect_y = 90, 3
        self.settings_button_rect_width, self.settings_button_rect_height = 80, 30
        self.settings_button_text = "设置"
        self.settings_button_color, self.settings_button_text_color = (50, 50, 50), (255, 255, 255)

        # save button
        self.save_button_rect_x, self.save_button_rect_y = 175, 3
        self.save_button_rect_width, self.save_button_rect_height = 80, 30
        self.save_button_text = "保存"
        self.save_button_color, self.save_button_text_color = (50, 50, 50), (255, 255, 255)

        # open button
        self.open_button_rect_x, self.open_button_rect_y = 260, 3
        self.open_button_rect_width, self.open_button_rect_height = 80, 30
        self.open_button_text = "打开"
        self.open_button_color, self.open_button_text_color = (50, 50, 50), (255, 255, 255)

        # frame add button
        self.frame_add_button_rect_x, self.frame_add_button_rect_y = self.animation_engine_window.get_width() - 50, self.animation_engine_window.get_height() - 25
        self.frame_add_button_rect_width, self.frame_add_button_rect_height = 20, 20
        self.frame_add_button_text = "+"
        self.frame_add_button_color, self.frame_add_button_text_color = (50, 50, 50), (255, 255, 255)

        # frame remove button
        self.frame_remove_button_rect_x, self.frame_remove_button_rect_y = self.animation_engine_window.get_width() - 25, self.animation_engine_window.get_height() - 25
        self.frame_remove_button_rect_width, self.frame_remove_button_rect_height = 20, 20
        self.frame_remove_button_text = "-"
        self.frame_remove_button_color, self.frame_remove_button_text_color = (50, 50, 50), (255, 255, 255)

        # frame set button
        self.frame_set_button_rect_x, self.frame_set_button_rect_y = self.animation_engine_window.get_width() - 100, self.animation_engine_window.get_height() - 25
        self.frame_set_button_rect_width, self.frame_set_button_rect_height = 40, 20
        self.frame_set_button_text = "set"
        self.frame_set_button_color, self.frame_set_button_text_color = (50, 50, 50), (255, 255, 255)




        self.WINDOW_WIDTH, self.WINDOW_HEIGHT = self.animation_engine_window.get_size()
        # 设置：显示区域在右下角
        self.grid_width = 320  # 宽度
        self.grid_height = 500  # 高度
        # 位置：右下角，留出边距
        self.margin = 20
        self.x = self.WINDOW_WIDTH - self.grid_width - self.margin
        self.y = self.WINDOW_HEIGHT - self.grid_height - self.margin - 15
        # 创建网格（2列显示）
        self.grid = image_scrolling.InfiniteImageGrid(
            x=self.x, 
            y=self.y, 
            width=self.grid_width, 
            height=self.grid_height,
            columns=2,          # 2列
            thumbnail_size=130, # 缩略图大小
            spacing=12          # 间距
        )

        self.icon_path = os.path.join(self.dirpath, "Image", "icon.ico")
        self.icon = pygame.image.load(self.icon_path)
        self.clock = pygame.time.Clock()
        pygame.display.set_icon(self.icon)
        self.running : bool = True
        while self.running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False

                if event.type == pygame.DROPFILE:
                    file_path = event.file  # 直接获取路径字符串
                    
                    if file_path.lower().endswith((".jpg", ".png", ".jpeg")):
                        self.grid.add_image(file_path)
                    else:
                        messagebox.showerror("Error", "Only support image files.(.jpg .png .jpeg)")


                self.mouse_pos = pygame.mouse.get_pos()
                
                if event.type == pygame.MOUSEBUTTONDOWN:
                    if event.button == 1:
                        # 只有点击空白区域才退出模式
                        if not self._is_clicking_on_control_point():
                            for key in list(self.is_rotateing.keys()):
                                self.is_rotateing[key] = False
                            for key in list(self.is_sizing.keys()):
                                self.is_sizing[key] = False


                    # Button click
                    if event.button == 1:
                        if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                                    self.import_button_rect_x, self.import_button_rect_y, self.import_button_rect_width, self.import_button_rect_height):
                            _import_button(self.grid)

                        if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                                    self.settings_button_rect_x, self.settings_button_rect_y, self.settings_button_rect_width, self.settings_button_rect_height):
                            if not _is_tk:
                                _settings_button()
                        if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                                        self.save_button_rect_x, self.save_button_rect_y, self.save_button_rect_width, self.save_button_rect_height):
                            self._save_to_file()
                        if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                                        self.open_button_rect_x, self.open_button_rect_y, self.open_button_rect_width, self.open_button_rect_height):
                            self._open_from_file()

                        if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                                    self.frame_add_button_rect_x, self.frame_add_button_rect_y, self.frame_add_button_rect_width, self.frame_add_button_rect_height):
                            self._update_every_about_frame_data()
                            _data["frame"] += 1
                            self._load_frame_data(_data["frame"], clear_first=True, copy_from_previous=True)

                        if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                                    self.frame_remove_button_rect_x, self.frame_remove_button_rect_y, self.frame_remove_button_rect_width, self.frame_remove_button_rect_height):
                            if _data["frame"] > 1:
                                self._update_every_about_frame_data()
                                _data["frame"] -= 1
                                self._load_frame_data(_data["frame"], clear_first=True, copy_from_previous=False)
                                
                        if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                                        self.frame_set_button_rect_x, self.frame_set_button_rect_y, self.frame_set_button_rect_width, self.frame_set_button_rect_height):
                            self._frame_set_button()
                    if event.button == 3:  # 右键
                        hover_index = self.grid.get_hover_image_index(self.mouse_pos[0], self.mouse_pos[1])
                        if hover_index != -1:
                            self._attribute_window(hover_index)
                    

                if event.type == pygame.MOUSEWHEEL:
                    if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                                   0, self.animation_engine_window.get_height()-35, self.animation_engine_window.get_width(), 35):
                        if event.y > 0:
                            self._update_every_about_frame_data()
                            _data["frame"] += 1
                            self._load_frame_data(_data["frame"], clear_first=True, copy_from_previous=True)
                        elif event.y < 0:
                            if _data["frame"] > 1:
                                self._update_every_about_frame_data()
                                _data["frame"] -= 1
                                self._load_frame_data(_data["frame"], clear_first=True, copy_from_previous=False)

                if event.type == pygame.MOUSEBUTTONUP:
                    # 重置缩放状态（只重置状态标志，不删除整个state）
                    if hasattr(self, 'size_state'):
                        for state in self.size_state.values():
                            state['resizing_x'] = False
                            state['resizing_y'] = False



                    # add image
                    if self.grid.dragging and event.button == 1:
                        drag_index = self.grid.drag_index
                        if drag_index != -1:
                            global ae_next_id
                            # 检查是否在编辑器区域内（左侧）
                            editor_rect = pygame.Rect(0, self.settings_rect_height, 
                                                    self.animation_engine_window.get_width() - self.grid_width - 19, 
                                                    780)
                            if editor_rect.collidepoint(self.mouse_pos):
                                # 生成新的唯一ID
                                new_id = ae_next_id
                                ae_next_id += 1
                                
                                # 保存新实例
                                ae_images_position_x[new_id] = self.grid.get_drag_image_rect(drag_index).x
                                ae_images_position_y[new_id] = self.grid.get_drag_image_rect(drag_index).y
                                self.is_has_image_drag[new_id] = False
                                self.is_has_image_display_attribution[new_id] = False
                                self.is_rotateing[new_id] = False
                                self.is_sizing[new_id] = False
                                ae_images_path[new_id] = self.grid.get_image_path(drag_index)
                                ae_images_direction[new_id] = 0
                                ae_images_size_x[new_id] = self.grid.original_images_size[drag_index][0]
                                ae_images_size_y[new_id] = self.grid.original_images_size[drag_index][1]
                                ae_images_id[drag_index] = new_id
                                ae_images_type[new_id] = "Image"
                                #self.grid.add_image_name(self.grid.get_image_path(drag_index))
                                

                self.grid.handle_event(event)
            


            pygame.display.set_caption(f"Animation Engine - {_data["animation_name"]}")

            self.animation_engine_window.fill((16, 16, 16))

            self.settings_rect_width, self.settings_rect_height = self.animation_engine_window.get_width(), 35
            self.frame_rect_height = 35

            # 编辑器区域
            self.editor_rect_x, self.editor_rect_y = 0, self.settings_rect_height
            self.editor_rect_width = self.animation_engine_window.get_width() - self.grid_width - 19
            self.editor_rect_height = self.animation_engine_window.get_height() - self.frame_rect_height
            pygame.draw.rect(
                self.animation_engine_window, 
                (8, 8, 8), 
                (
                self.editor_rect_x, 
                self.editor_rect_y, 
                self.editor_rect_width, 
                self.editor_rect_height
                )
            )

            self._update_editor()
            self._interactive_images()
            self._update_every_about_frame_data()

            # 设置栏
            pygame.draw.rect(
                self.animation_engine_window, 
                (100, 100, 100), 
                (
                    self.animation_engine_window.get_width() // 2 - self.settings_rect_width // 2,  # 矩形左上角的 x 坐标
                    0,  # 矩形左上角的 y 坐标，即窗口顶部
                    self.settings_rect_width,  # 矩形的宽度
                    self.settings_rect_height  # 矩形的高度
                )
            )

            # 帧栏
            pygame.draw.rect(
                self.animation_engine_window, 
                (100, 100, 100), 
                (
                    0,  # x
                    self.animation_engine_window.get_height()-self.frame_rect_height,  # y
                    self.animation_engine_window.get_width(),  # w
                    self.frame_rect_height # h
                )
            )





            # display import button
            _display_button(
                self.animation_engine_window, 
                self.font_path, 
                self.import_button_rect_x, 
                self.import_button_rect_y, 
                self.import_button_rect_width, 
                self.import_button_rect_height, 
                self.import_button_text, 
                self.import_button_color, 
                self.import_button_text_color
            )
            # display settings button
            _display_button(
                self.animation_engine_window, 
                self.font_path, 
                self.settings_button_rect_x, 
                self.settings_button_rect_y, 
                self.settings_button_rect_width, 
                self.settings_button_rect_height, 
                self.settings_button_text, 
                self.settings_button_color, 
                self.settings_button_text_color
            )
            # display frame add button
            _display_button(
                self.animation_engine_window, 
                self.font_path, 
                self.frame_add_button_rect_x, 
                self.frame_add_button_rect_y, 
                self.frame_add_button_rect_width, 
                self.frame_add_button_rect_height, 
                self.frame_add_button_text, 
                self.frame_add_button_color, 
                self.frame_add_button_text_color
            )
            # display save button
            _display_button(
                self.animation_engine_window, 
                self.font_path, 
                self.save_button_rect_x, 
                self.save_button_rect_y, 
                self.save_button_rect_width, 
                self.save_button_rect_height, 
                self.save_button_text, 
                self.save_button_color, 
                self.save_button_text_color, 
            )
            # display open button
            _display_button(
                self.animation_engine_window, 
                self.font_path, 
                self.open_button_rect_x, 
                self.open_button_rect_y, 
                self.open_button_rect_width, 
                self.open_button_rect_height, 
                self.open_button_text, 
                self.open_button_color, 
                self.open_button_text_color
            )


            #display frame remove button
            _display_button(
                self.animation_engine_window, 
                self.font_path, 
                self.frame_remove_button_rect_x, 
                self.frame_remove_button_rect_y, 
                self.frame_remove_button_rect_width, 
                self.frame_remove_button_rect_height, 
                self.frame_remove_button_text, 
                self.frame_remove_button_color, 
                self.frame_remove_button_text_color
            )
            #display frame set button
            _display_button(
                self.animation_engine_window, 
                self.font_path, 
                self.frame_set_button_rect_x, 
                self.frame_set_button_rect_y, 
                self.frame_set_button_rect_width, 
                self.frame_set_button_rect_height, 
                self.frame_set_button_text, 
                self.frame_set_button_color, 
                self.frame_set_button_text_color
            )


            # display frame value
            _display_text(
                self.animation_engine_window, 
                self.font_path, 
                70, 
                self.animation_engine_window.get_height() - 20, 
                f"Frame: {_data["frame"]}", 
                (255, 255, 255)
            )
            
            self.grid.draw(self.animation_engine_window, self.font_path)
            
            pygame.display.flip()
            self.clock.tick(self.FPS)


    def _save_to_file(self):
        """保存当前动画数据到文件"""
        file_path = filedialog.asksaveasfilename(
            title="保存动画",
            defaultextension=".animationdata",
            filetypes=[("动画数据文件", "*.animationdata"), ("所有文件", "*.*")], 
            initialfile=_data["animation_name"]
        )
        
        if not file_path:
            return
        
        # 编辑器区域尺寸（用于坐标缩放）
        editor_width = self.editor_rect_width
        editor_height = self.editor_rect_height
        
        # 构建 frame 数据
        frame_data = {}
        frame_data["1;" + str(_data["frame"])] = {"sprite_number": len(ae_images_path)}
        
        # 收集所有对象
        objects = {}
        obj_index = 1
        
        for key in ae_images_path:
            path = ae_images_path.get(key, "")
            if not path:
                continue
            
            # 找到这个图片ID对应的网格索引
            grid_index = None
            for idx, img_id in ae_images_id.items():
                if img_id == key:
                    grid_index = idx
                    break
            
            if grid_index is None:
                continue
            
            # 构建每个帧的数据
            frame_detail = {}
            show_frames = []
            
            for frame in range(1, _data["frame"] + 1):
                frame_path_str = ae_every_frame_path.get(frame, "")
                if not frame_path_str:
                    continue
                
                frame_path_list = frame_path_str.split(";")
                
                if grid_index < len(frame_path_list) and frame_path_list[grid_index]:
                    # 获取原始坐标
                    x = float(_get_value(ae_every_frame_position_x.get(frame, ""), grid_index, "0"))
                    y = float(_get_value(ae_every_frame_position_y.get(frame, ""), grid_index, "0"))
                    
                    # 转换为相对坐标（0-1范围），便于在不同分辨率下还原
                    rel_x = x / editor_width if editor_width > 0 else 0
                    rel_y = y / editor_height if editor_height > 0 else 0
                    
                    direction = float(_get_value(ae_every_frame_direction.get(frame, ""), grid_index, "0"))
                    size_x = float(_get_value(ae_every_frame_size_x.get(frame, ""), grid_index, "100"))
                    size_y = float(_get_value(ae_every_frame_size_y.get(frame, ""), grid_index, "100"))
                    
                    frame_detail[str(frame)] = {
                        "position": {"rel_x": rel_x, "rel_y": rel_y},
                        "direction": direction,
                        "size": {"size_x": size_x, "size_y": size_y}
                    }
                    show_frames.append(str(frame))
            
            if frame_detail:
                objects[f"Object{obj_index}"] = {
                    "name": f"{os.path.basename(path).split('.')[0]}",
                    "type": ae_images_type.get(key, "Image"),
                    "path": path,
                    "show_frame": ";".join(show_frames),
                    "frame": frame_detail
                }
                obj_index += 1
        
        # 构建完整JSON
        output = {
            "name": _data["animation_name"],
            "total_frame": _data["frame"],
            "editor_width": editor_width,
            "editor_height": editor_height,
            "frame": frame_data,
            **objects
        }
        
        with open(file_path, "w", encoding="utf-8") as f:
            json.dump(output, f, indent=2, ensure_ascii=False)
        
        messagebox.showinfo("成功", f"动画已保存到:\n{file_path}")


    def _open_from_file(self):
        """从文件打开动画"""
        file_path = filedialog.askopenfilename(
            title="打开动画",
            filetypes=[("动画数据文件", "*.animationdata"), ("所有文件", "*.*")]
        )
        
        if not file_path:
            return
        
        with open(file_path, "r", encoding="utf-8") as f:
            data = json.load(f)
        
        # 设置动画名称
        _data["animation_name"] = data.get("name", "untitled")
        
        # 获取总帧数（从frame的key中解析）
        frame_keys = list(data.get("frame", {}).keys())
        if frame_keys:
            # 解析 "1;2" 格式，取最后一个数字
            last_frame = frame_keys[0].split(";")[-1]
            _data["frame"] = int(last_frame)
        
        # 清空当前所有图片
        for key in list(ae_images_path.keys()):
            self._delete_image(key, ask_confirm=False)
        
        # 加载对象
        global ae_next_id
        for obj_name, obj_data in data.items():
            if obj_name.startswith("Object"):
                new_id = ae_next_id
                ae_next_id += 1
                
                path = obj_data.get("path", "")
                ae_images_path[new_id] = path
                ae_images_type[new_id] = obj_data.get("type", "Image")
                
                # 获取show_frame，解析该对象出现在哪些帧
                show_frame_str = obj_data.get("show_frame", "")
                show_frames = show_frame_str.split(";") if show_frame_str else []
                
                # 加载每一帧的数据
                frame_detail = obj_data.get("frame", {})
                
                for frame_str, frame_info in frame_detail.items():
                    frame = int(frame_str)
                    pos = frame_info.get("position", {})
                    
                    # 兼容两种格式
                    if "rel_x" in pos and "rel_y" in pos:
                        # 相对坐标格式，转换为绝对坐标
                        editor_w = data.get("editor_width", 1261)
                        editor_h = data.get("editor_height", 780)
                        x = pos["rel_x"] * editor_w
                        y = pos["rel_y"] * editor_h
                    elif "x" in pos and "y" in pos:
                        x = pos["x"]
                        y = pos["y"]
                    else:
                        x, y = 0, 0
                    
                    direction = frame_info.get("direction", 0)
                    size = frame_info.get("size", {"size_x": 100, "size_y": 100})
                    
                    # 确保该帧的数据字典存在
                    if frame not in ae_every_frame_position_x:
                        ae_every_frame_position_x[frame] = ""
                        ae_every_frame_position_y[frame] = ""
                        ae_every_frame_path[frame] = ""
                        ae_every_frame_direction[frame] = ""
                        ae_every_frame_size_x[frame] = ""
                        ae_every_frame_size_y[frame] = ""
                        ae_every_frame_type[frame] = ""
                    
                    # 获取现有的字符串
                    pos_x_str = ae_every_frame_position_x[frame]
                    pos_y_str = ae_every_frame_position_y[frame]
                    path_str = ae_every_frame_path[frame]
                    dir_str = ae_every_frame_direction[frame]
                    size_x_str = ae_every_frame_size_x[frame]
                    size_y_str = ae_every_frame_size_y[frame]
                    type_str = ae_every_frame_type[frame]
                    
                    # 追加新数据
                    # 追加新数据
                    if pos_x_str:
                        ae_every_frame_position_x[frame] = pos_x_str + ";" + str(x)
                        ae_every_frame_position_y[frame] = pos_y_str + ";" + str(y)
                        ae_every_frame_path[frame] = path_str + ";" + path
                        ae_every_frame_direction[frame] = dir_str + ";" + str(direction)
                        ae_every_frame_size_x[frame] = size_x_str + ";" + str(size["size_x"])
                        ae_every_frame_size_y[frame] = size_y_str + ";" + str(size["size_y"])
                        ae_every_frame_type[frame] = type_str + ";" + obj_data.get("type", "Image")
                    else:
                        ae_every_frame_position_x[frame] = str(x)
                        ae_every_frame_position_y[frame] = str(y)
                        ae_every_frame_path[frame] = path
                        ae_every_frame_direction[frame] = str(direction)
                        ae_every_frame_size_x[frame] = str(size["size_x"])
                        ae_every_frame_size_y[frame] = str(size["size_y"])
                        ae_every_frame_type[frame] = obj_data.get("type", "Image")
                
                # 初始化状态
                self.is_has_image_drag[new_id] = False
                self.is_has_image_display_attribution[new_id] = False
                self.is_rotateing[new_id] = False
                self.is_sizing[new_id] = False
        
        # 重新加载当前帧
        self._load_frame_data(_data["frame"])
        
        messagebox.showinfo("成功", f"已打开动画:\n{file_path}")

    def _frame_set_button(self):
        frame = simpledialog.askstring("设置帧数", "请输入帧数：")
        if frame:
            try:
                frame = int(frame)
                if frame > 0:
                    _data["frame"] = frame
                    self._load_frame_data(frame)
                    return
                else:
                    messagebox.showerror("Error", "帧数必须大于0！")
            except ValueError:
                messagebox.showerror("Error", "请输入正确的数字！")
            except TypeError:
                messagebox.showerror("Error", "请输入数字！")
            self._frame_set_button()

    def _is_clicking_on_control_point(self):
        """检查鼠标是否点击在任何图片的控制点上"""
        for key in ae_images_path:
            x = ae_images_position_x.get(key, 0)
            y = ae_images_position_y.get(key, 0)
            w = ae_images_size_x.get(key, self.grid.thumbnail_size)
            h = ae_images_size_y.get(key, self.grid.thumbnail_size)
            
            center_x = x + w // 2
            center_y = y + h // 2
            control_distance = 100
            x_control_x = center_x + control_distance
            y_control_y = center_y + control_distance
            
            # 检查X控制点
            if _collisions(self.mouse_pos[0], self.mouse_pos[1], 1, 1,
                        x_control_x - 5, center_y - 5, 10, 10):
                return True
            # 检查Y控制点
            if _collisions(self.mouse_pos[0], self.mouse_pos[1], 1, 1,
                        center_x - 5, y_control_y - 5, 10, 10):
                return True
        return False
    

    def _add_attribute_choose(self, x, y, choose_height, choose_width, *texts):
        """添加属性选择"""
        pygame.draw.rect(
            self.animation_engine_window, 
            (100, 100, 100), 
            (
                x, y, 
                choose_width, 30 * len(texts)
            )
        )
        for i, text in enumerate(texts):
            _display_button(
                self.animation_engine_window, 
                self.font_path, 
                x + 5, y + 5 + choose_height * i, 
                choose_width - 5, choose_height - 5, 
                text, 
                (80, 80, 80), 
                (255, 255, 255), 
                16
            )
            if _collisions(self.mouse_pos[0], self.mouse_pos[1], 10, 10, 
                        x+5, y+5 + choose_height * i, choose_width-5,  choose_height-5):
                if pygame.mouse.get_pressed()[0]:
                    return text

    def _rebuild_dirty_caches(self):
        """重建所有脏缓存"""
        for image_index in list(self._dirty_caches):
            path = ae_images_path.get(image_index, "")
            w = ae_images_size_x.get(image_index, self.grid.thumbnail_size)
            h = ae_images_size_y.get(image_index, self.grid.thumbnail_size)
            direction = ae_images_direction.get(image_index, 0)
            
            # 删除旧缓存
            cache_key = (path, w, h, direction)
            if cache_key in self.editor_images_cache:
                del self.editor_images_cache[cache_key]
            
            # 新缓存会在 _update_editor 中自动生成
        self._dirty_caches.clear()


    def _update_editor(self):
        """绘制编辑器中的图片"""
        self._rebuild_dirty_caches()
        
        for key in ae_images_path:
            x = ae_images_position_x.get(key, 0)
            y = ae_images_position_y.get(key, 0)
            w = ae_images_size_x.get(key, self.grid.thumbnail_size)
            h = ae_images_size_y.get(key, self.grid.thumbnail_size)
            direction = ae_images_direction.get(key, 0)
            path = ae_images_path.get(key, "")
            
            if not path or not os.path.exists(path):
                continue
            
            # 计算缩放比例
            original_image = pygame.image.load(path)
            orig_w, orig_h = original_image.get_size()
            scale_x = w / orig_w
            scale_y = h / orig_h
            
            # 使用 rotozoom 一次性处理旋转和缩放
            cache_key = (path, direction, w, h)
            if cache_key not in self.editor_images_cache:
                # pygame.transform.rotozoom(image, angle, scale)
                # 注意：rotozoom 的 scale 是等比例缩放
                # 如果需要非等比例，需要特殊处理
                scale = min(scale_x, scale_y)  # 等比例缩放，保持宽高比
                rotated_scaled = pygame.transform.rotozoom(original_image, direction, scale)
                self.editor_images_cache[cache_key] = rotated_scaled
            
            image_to_draw = self.editor_images_cache[cache_key]
            rect = image_to_draw.get_rect(center=(x + w//2, y + h//2))
            self.animation_engine_window.blit(image_to_draw, rect)

    def _update_every_about_frame_data(self):
        """更新当前帧的所有图片数据到每帧数据字典"""
        current_frame = _data["frame"]
        
        # 收集当前所有图片的数据
        positions_x = []
        positions_y = []
        paths = []
        directions = []
        sizes_x = []
        sizes_y = []
        types = []
        
        for key in ae_images_path:
            positions_x.append(str(ae_images_position_x.get(key, 0)))
            positions_y.append(str(ae_images_position_y.get(key, 0)))
            paths.append(ae_images_path.get(key, ""))
            directions.append(str(ae_images_direction.get(key, 0)))
            sizes_x.append(str(ae_images_size_x.get(key, 100)))
            sizes_y.append(str(ae_images_size_y.get(key, 100)))
            types.append(ae_images_type.get(key, "Image"))
        
        # 用分号连接所有数据
        ae_every_frame_position_x[current_frame] = ";".join(positions_x) if positions_x else ""
        ae_every_frame_position_y[current_frame] = ";".join(positions_y) if positions_y else ""
        ae_every_frame_path[current_frame] = ";".join(paths) if paths else ""
        ae_every_frame_direction[current_frame] = ";".join(directions) if directions else ""
        ae_every_frame_size_x[current_frame] = ";".join(sizes_x) if sizes_x else ""
        ae_every_frame_size_y[current_frame] = ";".join(sizes_y) if sizes_y else ""
        ae_every_frame_type[current_frame] = ";".join(types) if types else ""
        

    def _load_frame_data(self, frame, clear_first=True, copy_from_previous=False):
        """加载指定帧的数据"""
        global ae_next_id
        
        # 如果需要从上一帧复制数据，且当前帧没有数据
        if copy_from_previous and frame > 1:
            # 检查当前帧是否已有数据
            current_path_str = ae_every_frame_path.get(frame, "")
            if not current_path_str:
                prev_frame = frame - 1
                prev_path_str = ae_every_frame_path.get(prev_frame, "")
                if prev_path_str:
                    # 复制上一帧的数据到当前帧
                    ae_every_frame_position_x[frame] = ae_every_frame_position_x.get(prev_frame, "")
                    ae_every_frame_position_y[frame] = ae_every_frame_position_y.get(prev_frame, "")
                    ae_every_frame_path[frame] = prev_path_str
                    ae_every_frame_direction[frame] = ae_every_frame_direction.get(prev_frame, "")
                    ae_every_frame_size_x[frame] = ae_every_frame_size_x.get(prev_frame, "")
                    ae_every_frame_size_y[frame] = ae_every_frame_size_y.get(prev_frame, "")
                    ae_every_frame_type[frame] = ae_every_frame_type.get(prev_frame, "")
        
        # 获取该帧的数据字符串
        pos_x_str = ae_every_frame_position_x.get(frame, "")
        pos_y_str = ae_every_frame_position_y.get(frame, "")
        path_str = ae_every_frame_path.get(frame, "")
        dir_str = ae_every_frame_direction.get(frame, "")
        size_x_str = ae_every_frame_size_x.get(frame, "")
        size_y_str = ae_every_frame_size_y.get(frame, "")
        
        if not path_str:
            if clear_first:
                for key in list(ae_images_path.keys()):
                    self._delete_image(key, ask_confirm=False)
            return
        
        if clear_first:
            # 清空当前所有图片
            for key in list(ae_images_path.keys()):
                self._delete_image(key, ask_confirm=False)
            
            # 清空 ae_images_id
            ae_images_id.clear()
            
            # 分割字符串
            pos_x_list = pos_x_str.split(";")
            pos_y_list = pos_y_str.split(";")
            path_list = path_str.split(";")
            dir_list = dir_str.split(";")
            size_x_list = size_x_str.split(";")
            size_y_list = size_y_str.split(";")
            
            # 重新创建图片数据
            for i in range(len(path_list)):
                if i >= len(path_list) or not path_list[i]:
                    continue
                
                new_id = ae_next_id
                ae_next_id += 1
                
                ae_images_path[new_id] = path_list[i]
                ae_images_position_x[new_id] = float(pos_x_list[i]) if i < len(pos_x_list) else 0
                ae_images_position_y[new_id] = float(pos_y_list[i]) if i < len(pos_y_list) else 0
                ae_images_direction[new_id] = float(dir_list[i]) if i < len(dir_list) else 0
                ae_images_size_x[new_id] = float(size_x_list[i]) if i < len(size_x_list) else 100
                ae_images_size_y[new_id] = float(size_y_list[i]) if i < len(size_y_list) else 100
                ae_images_type[new_id] = "Image"
                
                # 关键：设置 ae_images_id，使用索引 i 作为网格索引
                ae_images_id[i] = new_id
                
                # 初始化状态
                self.is_has_image_drag[new_id] = False
                self.is_has_image_display_attribution[new_id] = False
                self.is_rotateing[new_id] = False
                self.is_sizing[new_id] = False

    def _delete_image(self, image_key, ask_confirm=True):
        """删除图片及其所有相关数据"""
        image_path = ae_images_path.get(image_key, "")
        if ask_confirm:
            image_name = os.path.basename(image_path) if image_path else f"图片{image_key}"
            result = messagebox.askyesno(
                "确认删除",
                f"确定要删除图片「{image_name}」吗？\n\n删除后无法恢复！",
                icon='warning'
            )
            if not result:
                return
        
        # 1. 清除缓存中的该图片
        keys_to_delete = []
        for cache_key in self.editor_images_cache:
            if cache_key[0] == image_path:
                keys_to_delete.append(cache_key)
        for key in keys_to_delete:
            del self.editor_images_cache[key]
        
        # 2. 清除状态字典
        self.is_has_image_drag.pop(image_key, None)
        self.is_has_image_display_attribution.pop(image_key, None)
        self.is_rotateing.pop(image_key, None)
        self.is_sizing.pop(image_key, None)
        
        # 3. 清除缩放状态
        if hasattr(self, 'size_state') and image_key in self.size_state:
            del self.size_state[image_key]
        
        # 4. 清除全局数据字典
        ae_images_position_x.pop(image_key, None)
        ae_images_position_y.pop(image_key, None)
        ae_images_path.pop(image_key, None)
        ae_images_direction.pop(image_key, None)
        ae_images_size_x.pop(image_key, None)
        ae_images_size_y.pop(image_key, None)
        ae_images_type.pop(image_key, None)


    def _interactive_images(self):
        """交互式图片编辑"""
        for key in list(ae_images_path.keys()):
            x = ae_images_position_x.get(key, 0)
            y = ae_images_position_y.get(key, 0)
            w = ae_images_size_x.get(key, self.grid.thumbnail_size)
            h = ae_images_size_y.get(key, self.grid.thumbnail_size)
            
            # 检查鼠标是否在图片上或正在拖动
            is_hover = _collisions(self.mouse_pos[0], self.mouse_pos[1], 1, 1, x, y, w, h)
            
            # 如果正在拖动其他图片，跳过当前图片的拖动逻辑
            if self.is_dragging and self.current_dragging_key != key:
                # 但仍然需要处理旋转和缩放模式
                if self.is_rotateing.get(key, False):
                    self._rotate_direction(key)
                if self.is_sizing.get(key, False):
                    self._size(key)
                continue
            
            if is_hover or self.is_has_image_drag.get(key, False) or self.is_has_image_display_attribution.get(key, False):
                if not self.is_rotateing.get(key, False) and not self.is_sizing.get(key, False):
                    # 绘制高亮边框
                    pygame.draw.rect(self.animation_engine_window, self.YELLOW, (x, y, w, h), 2)
                    
                    # 右键弹出属性选择
                    if pygame.mouse.get_pressed()[2] or self.is_has_image_display_attribution.get(key, False):
                        global old_mouse_pos
                        if not self.is_has_image_display_attribution.get(key, False):
                            old_mouse_pos = self.mouse_pos
                        self.is_has_image_display_attribution[key] = True

                        choose = self._add_attribute_choose(old_mouse_pos[0], old_mouse_pos[1], 30, 200, "属性", "打开图片所在路径", "旋转", "大小", "删除")
                        if choose == "属性":
                            # 方法1：反向查找 - 从 ae_images_id 中找对应的键
                            grid_index = None
                            for idx, img_id in ae_images_id.items():
                                if img_id == key:
                                    grid_index = idx
                                    break
                            
                            # 方法2：如果方法1找不到，直接从网格的原始数据中查找
                            if grid_index is None:
                                # 尝试直接从网格中找
                                for idx in range(len(self.grid.image_paths)):
                                    if self.grid.image_paths[idx] == ae_images_path.get(key, ""):
                                        grid_index = idx
                                        break
                            
                            if grid_index is not None:
                                self._attribute_window(grid_index)
                            else:
                                # 如果还是找不到，直接用 key 作为 grid_index 试试
                                if key < len(self.grid.image_paths):
                                    self._attribute_window(key)
                                else:
                                    messagebox.showerror("错误", f"找不到图片索引，图片ID: {key}")
                        elif choose == "打开图片所在路径":
                            dirpath = os.path.dirname(ae_images_path.get(key, ""))
                            if dirpath:
                                if sys.platform == "win32":
                                    os.startfile(dirpath)
                                elif sys.platform == "darwin":
                                    subprocess.run(["open", dirpath])
                                else:
                                    subprocess.run(["xdg-open", dirpath])
                        elif choose == "旋转":
                            self.is_rotateing[key] = True
                            # 退出缩放模式
                            self.is_sizing[key] = False
                        elif choose == "大小":
                            self.is_sizing[key] = True
                            # 退出旋转模式
                            self.is_rotateing[key] = False
                        elif choose == "删除":
                            self._delete_image(key)
                            return  # 删除后立即返回，避免继续处理已删除的图片
                    
                    # 左键拖动
                    if pygame.mouse.get_pressed()[0]:
                        if self.is_has_image_display_attribution.get(key, False):
                            self.is_has_image_display_attribution[key] = False
                            continue
                        
                        # 开始拖动
                        if not self.is_has_image_drag.get(key, False) and not self.is_dragging:
                            self.is_has_image_drag[key] = True
                            self.is_dragging = True
                            self.current_dragging_key = key
                        
                        # 只有当前拖动的图片才能移动
                        if self.is_dragging and self.current_dragging_key == key:
                            # 编辑器区域边界
                            editor_left = 0
                            editor_top = self.settings_rect_height
                            editor_right = self.animation_engine_window.get_width() - self.grid_width - 19
                            editor_bottom = self.animation_engine_window.get_height() - self.frame_rect_height
                            
                            # 计算新位置（图片中心跟随鼠标）
                            new_x = self.mouse_pos[0] - w // 2
                            new_y = self.mouse_pos[1] - h // 2
                            
                            # 限制在编辑器区域内
                            new_x = max(editor_left, min(new_x, editor_right - w))
                            new_y = max(editor_top, min(new_y, editor_bottom - h))
                            
                            ae_images_position_x[key] = new_x
                            ae_images_position_y[key] = new_y
                    else:
                        # 鼠标松开，结束拖动
                        if self.is_has_image_drag.get(key, False):
                            self.is_has_image_drag[key] = False
                            self.is_dragging = False
                            self.current_dragging_key = None
            
            # 处理旋转和缩放模式（即使不在hover状态也要处理）
            if self.is_rotateing.get(key, False):
                self._rotate_direction(key)
            if self.is_sizing.get(key, False):
                self._size(key)

    def _rotate_direction(self, image_index):
        """旋转图片方向 - 只更新角度，不实时重建缓存"""
        actual_size_x = ae_images_size_x.get(image_index, 100)
        actual_size_y = ae_images_size_y.get(image_index, 100)
        radius = min(actual_size_x, actual_size_y) + 100
        
        center_x = ae_images_position_x.get(image_index, 0) + actual_size_x // 2
        center_y = ae_images_position_y.get(image_index, 0) + actual_size_y // 2
        
        pygame.draw.circle(
            self.animation_engine_window, 
            "green", 
            (center_x, center_y), 
            radius=radius, 
            width=3
        )
        
        dx = self.mouse_pos[0] - center_x
        dy = self.mouse_pos[1] - center_y
        distance = math.hypot(dx, dy)
        
        if distance <= radius:
            if distance > 0:
                line_length = min(distance, radius * 0.8)
                direction_x = dx / distance
                direction_y = dy / distance
                end_x = center_x + direction_x * line_length
                end_y = center_y + direction_y * line_length
            else:
                end_x, end_y = center_x, center_y
            
            pygame.draw.line(
                self.animation_engine_window, 
                "red", 
                (center_x, center_y), 
                (end_x, end_y), 
                3
            )
            
            pygame.draw.circle(
                self.animation_engine_window, 
                "yellow", 
                (int(end_x), int(end_y)), 
                8
            )
            
            if distance > 0:
                angle = math.degrees(math.atan2(dy, dx))
                final_angle = -angle
                ae_images_direction[image_index] = final_angle
                # 标记缓存为脏
                self._dirty_caches.add(image_index)
        else:
            if distance > 0:
                direction_x = dx / distance
                direction_y = dy / distance
                end_x = center_x + direction_x * radius * 0.8
                end_y = center_y + direction_y * radius * 0.8
                
                pygame.draw.line(
                    self.animation_engine_window, 
                    (100, 100, 100),
                    (center_x, center_y), 
                    (end_x, end_y), 
                    2
                )
                
                pygame.draw.circle(
                    self.animation_engine_window, 
                    (100, 100, 100),
                    (int(end_x), int(end_y)), 
                    6
                )


    def _size(self, image_index):
        """调整图片大小"""
        x = ae_images_position_x.get(image_index, 0)
        y = ae_images_position_y.get(image_index, 0)
        w = ae_images_size_x.get(image_index, self.grid.thumbnail_size)
        h = ae_images_size_y.get(image_index, self.grid.thumbnail_size)
        
        center_x = x + w // 2
        center_y = y + h // 2
        
        pygame.draw.rect(self.animation_engine_window, self.YELLOW, (x, y, w, h), 2)
        
        control_distance = 100
        x_control_x = center_x + control_distance
        y_control_y = center_y + control_distance
        
        if not hasattr(self, 'size_state'):
            self.size_state = {}
        
        if image_index not in self.size_state:
            self.size_state[image_index] = {
                'resizing_x': False,
                'resizing_y': False,
                'start_mouse_x': 0,
                'start_mouse_y': 0,
                'start_w': w,
                'start_h': h
            }
        
        state = self.size_state[image_index]
        
        pygame.draw.line(
            self.animation_engine_window, 
            (255, 0, 0), 
            (center_x, center_y), 
            (x_control_x, center_y), 
            2
        )
        pygame.draw.rect(
            self.animation_engine_window, 
            "red", 
            (x_control_x - 5, center_y - 5, 10, 10)
        )
        
        pygame.draw.line(
            self.animation_engine_window, 
            (0, 255, 0), 
            (center_x, center_y), 
            (center_x, y_control_y), 
            2
        )
        pygame.draw.rect(
            self.animation_engine_window, 
            "green", 
            (center_x - 5, y_control_y - 5, 10, 10)
        )
        
        if not state['resizing_x'] and not state['resizing_y']:
            mouse_on_x = _collisions(
                self.mouse_pos[0], self.mouse_pos[1], 1, 1,
                x_control_x - 5, center_y - 5, 10, 10
            )
            mouse_on_y = _collisions(
                self.mouse_pos[0], self.mouse_pos[1], 1, 1,
                center_x - 5, y_control_y - 5, 10, 10
            )
            
            if mouse_on_x and pygame.mouse.get_pressed()[0]:
                state['resizing_x'] = True
                state['start_mouse_x'] = self.mouse_pos[0]
                state['start_w'] = w
            
            if mouse_on_y and pygame.mouse.get_pressed()[0]:
                state['resizing_y'] = True
                state['start_mouse_y'] = self.mouse_pos[1]
                state['start_h'] = h
        
        if state['resizing_x']:
            delta_x = self.mouse_pos[0] - state['start_mouse_x']
            new_w = max(10, state['start_w'] + delta_x)
            ae_images_size_x[image_index] = new_w
            
            # 清除缩放缓存（只清除这个图片的缩放缓存）
            scale_cache_key = (ae_images_path.get(image_index, ""), w, h)
            if scale_cache_key in self.editor_images_cache:
                del self.editor_images_cache[scale_cache_key]
        
        if state['resizing_y']:
            delta_y = self.mouse_pos[1] - state['start_mouse_y']
            new_h = max(10, state['start_h'] + delta_y)
            ae_images_size_y[image_index] = new_h
            
            # 清除缩放缓存
            scale_cache_key = (ae_images_path.get(image_index, ""), w, h)
            if scale_cache_key in self.editor_images_cache:
                del self.editor_images_cache[scale_cache_key]




    # Window
    def _attribute_window(self, image_index : int):
        global _is_tk

        _is_tk = True

        loop = tk.Tk()
        loop.title(f"属性 - {self.grid.image_names[image_index]}")
        loop.geometry("800x600")
        loop.resizable(False, False)
        icon = tk.PhotoImage(file=os.path.join(os.path.dirname(__file__), "Image/tk_icon.gif"))
        loop.iconphoto(False, icon)
        loop.config(bg=RGB(16, 16, 16).of())

        # Text
        notice_text = tk.Label(loop, 
                            text="⚠所有设置关闭后生效！", 
                            font=(self.font_path, 16, "bold"), 
                            bg=RGB(16, 16, 16).of(), 
                            fg=RGB(255, 0, 0).of()
        )
        notice_text.pack(pady=10)
        name_text = tk.Label(loop, 
                            text=f"名称：{self.grid.image_names[image_index]}", 
                            font=(self.font_path, 16), 
                            bg=RGB(50, 50, 50).of(), 
                            fg=RGB(255, 255, 255).of()
        )
        name_text.pack(pady=10)
        path_text = tk.Label(loop, 
                            text=f"路径：{self.grid.image_paths[image_index]}", 
                            font=(self.font_path, 16), 
                            bg=RGB(50, 50, 50).of(), 
                            fg=RGB(255, 255, 255).of()
        )


        # Button
        def name_button():
            name = simpledialog.askstring("重命名", "请输入新的名称：", parent=loop)
            if name:
                self.grid.image_names[image_index] = name
                name_text.config(text=f"名称：{self.grid.image_names[image_index]}")
            
        name_button = tk.Button(loop, 
                                text="重命名", 
                                font=(self.font_path, 16), 
                                command=name_button, 
                                bg=RGB(50, 50, 50).of(), 
                                fg=RGB(255, 255, 255).of()
        )
        name_button.pack()

        
        path_text.pack(pady=10)

        loop.mainloop()
        _is_tk = False

    def _image_interactive_window(self, global_id : int):
        global _is_tk

        _is_tk = True

        loop = tk.Tk()
        # 从全局字典获取图片路径来显示名称
        image_path = ae_images_path.get(global_id, "")
        image_name = os.path.basename(image_path) if image_path else "未知"
        loop.title(f"属性 - {image_name}")
        loop.geometry("800x600")
        loop.resizable(False, False)
        icon = tk.PhotoImage(file=os.path.join(os.path.dirname(__file__), "Image/tk_icon.gif"))
        loop.iconphoto(False, icon)
        loop.config(bg=RGB(16, 16, 16).of())

        # 使用 global_id 访问字典
        name_text = tk.Label(loop, 
            text=f"名称：{image_name}", 
            font=(self.font_path, 16), 
            bg=RGB(16, 16, 16).of(), 
            fg=RGB(255, 255, 255).of()
        )
        name_text.pack()
        
        path_text = tk.Label(loop, 
            text=f"路径：{ae_images_path.get(global_id, '')}", 
            font=(self.font_path, 16), 
            bg=RGB(16, 16, 16).of(), 
            fg=RGB(255, 255, 255).of()
        )
        path_text.pack()
        
        position_text = tk.Label(loop, 
            text=f"对于窗口的坐标：({ae_images_position_x.get(global_id, 0)}, {ae_images_position_y.get(global_id, 0)})", 
            font=(self.font_path, 16), 
            bg=RGB(16, 16, 16).of(), 
            fg=RGB(255, 255, 255).of()
        )
        position_text.pack()
        
        size_text = tk.Label(loop, 
            text=f"大小：({ae_images_size_x.get(global_id, 100)}, {ae_images_size_y.get(global_id, 100)})", 
            font=(self.font_path, 16), 
            bg=RGB(16, 16, 16).of(), 
            fg=RGB(255, 255, 255).of()
        )
        size_text.pack()
        
        direction_text = tk.Label(loop, 
            text=f"方向：{ae_images_direction.get(global_id, 0)}", 
            font=(self.font_path, 16), 
            bg=RGB(16, 16, 16).of(), 
            fg=RGB(255, 255, 255).of()
        )
        direction_text.pack()

        loop.mainloop()
        _is_tk = False

        

if __name__ == '__main__':
    animation_engine = AnimationEngine()
    pygame.quit()


import pygame
import os
import sys
import image_scrolling
import math
import subprocess
from tkinter import messagebox, filedialog, simpledialog
import tkinter as tk
import json
from rgb import RGB
from data import (
    ae_images_position_x, 
    ae_images_position_y, 
    ae_images_path, 
    ae_images_direction, 
    ae_images_size_x, 
    ae_images_size_y, 
    ae_next_id, 
    ae_images_id, 
    ae_images_type, 
    ae_every_frame_position_x, 
    ae_every_frame_position_y, 
    ae_every_frame_path, 
    ae_every_frame_direction, 
    ae_every_frame_size_x, 
    ae_every_frame_size_y, 
    ae_every_frame_type
)

class AnimationEngine:
    """
    Just use:
        var = AnimationEngine()
    and that's it
    
    
    直接使用：
        var = AnimationEngine()
    就行了
    """
    def __init__(self):
        ...


import pygame
import sys
import math
import os

# 初始化
pygame.init()
WINDOW_WIDTH, WINDOW_HEIGHT = 1200, 800
screen = pygame.display.set_mode((WINDOW_WIDTH, WINDOW_HEIGHT))
pygame.display.set_caption("图片网格 - 无限滚动")
clock = pygame.time.Clock()

# 颜色
WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
GRAY = (200, 200, 200)
DARK_GRAY = (100, 100, 100)
BLUE = (70, 130, 230)
RED = (255, 100, 100)
LIGHT_BLUE = (173, 216, 230)

class ScrollBar:
    """垂直滚动条"""
    def __init__(self, x, y, height, content_height, view_height):
        self.rect = pygame.Rect(x, y, 16, height)
        self.content_height = content_height
        self.view_height = view_height
        self.dragging = False
        self.update_slider()
    
    def update_slider(self):
        """更新滑块大小和位置"""
        if self.content_height <= self.view_height:
            self.slider_height = self.rect.height
            self.slider_y = self.rect.y
            self.max_scroll = 0
            self.max_slider_move = 0
        else:
            ratio = self.view_height / self.content_height
            self.slider_height = max(30, int(self.rect.height * ratio))
            self.max_slider_move = self.rect.height - self.slider_height
            self.max_scroll = self.content_height - self.view_height
            if hasattr(self, 'scroll_ratio'):
                self.slider_y = self.rect.y + (self.scroll_ratio * self.max_slider_move)
            else:
                self.slider_y = self.rect.y
    
    def set_content_size(self, content_height, view_height):
        self.content_height = content_height
        self.view_height = view_height
        self.update_slider()
    
    def handle_event(self, event):
        if event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
            slider_rect = pygame.Rect(self.rect.x, self.slider_y, self.rect.width, self.slider_height)
            if slider_rect.collidepoint(event.pos):
                self.dragging = True
                self.drag_offset = event.pos[1] - self.slider_y
            elif self.rect.collidepoint(event.pos):
                self.slider_y = event.pos[1] - self.slider_height / 2
                self._clamp_slider()
                self.dragging = True
                self.drag_offset = self.slider_height / 2
                
        elif event.type == pygame.MOUSEBUTTONUP and event.button == 1:
            self.dragging = False
            
        elif event.type == pygame.MOUSEMOTION and self.dragging:
            self.slider_y = event.pos[1] - self.drag_offset
            self._clamp_slider()
    
    def _clamp_slider(self):
        self.slider_y = max(self.rect.y, min(self.slider_y, self.rect.y + self.max_slider_move))
    
    def get_scroll_offset(self):
        if self.content_height <= self.view_height:
            return 0
        
        slider_move = self.slider_y - self.rect.y
        self.scroll_ratio = slider_move / self.max_slider_move if self.max_slider_move > 0 else 0
        return int(self.scroll_ratio * self.max_scroll)
    
    def draw(self, screen):
        pygame.draw.rect(screen, (30, 30, 30), self.rect)
        pygame.draw.rect(screen, DARK_GRAY, self.rect, 1)
        slider_rect = pygame.Rect(self.rect.x, self.slider_y, self.rect.width, self.slider_height)
        color = BLUE if not self.dragging else RED
        pygame.draw.rect(screen, color, slider_rect)
        pygame.draw.rect(screen, DARK_GRAY, slider_rect, 1)

class InfiniteImageGrid:
    """无限滚动的图片网格（多列布局）"""
    def __init__(self, x, y, width, height, columns=2, thumbnail_size=120, spacing=15, name_height=20, bg_color=(30, 30, 30)):
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.columns = columns  # 列数（这里默认2行，实际是2列）
        self.thumbnail_size = thumbnail_size
        self.spacing = spacing
        self.bg_color = bg_color
        
        self.images = []  # 存储图片Surface
        self.image_paths = []  # 存储路径
        self.image_names = []  # 存储名称
        self.original_images = []  # 存储原始图片
        self.original_images_size = []  # 存储原始图片大小
        
        self.scroll_offset = 0
        self.total_content_height = 0
        
        # 滚动条在右侧
        scrollbar_x = x + width + 8
        scrollbar_y = y
        self.scrollbar = ScrollBar(scrollbar_x, scrollbar_y, height, 0, height)


        self.name_height = name_height  # 名字显示区域高度
    
        # 计算每行高度（图片高度 + 间距 + 名字高度）
        self.row_height = thumbnail_size + spacing + name_height
        
        # 裁剪表面
        self.clip_surface = pygame.Surface((width, height))


        # 拖动相关
        self.dragging = False  # 是否有图片被拖动属性
        self.drag_index = -1
        self.drag_start_pos = (0, 0)
        self.drag_start_image_pos = (0, 0)
        self.drag_current_offset = (0, 0)
        self.drag_offset_x = 0
        self.drag_offset_y = 0

    def add_image_name(self, image_path):
        """设置指定图片的名字"""
        name = os.path.basename(image_path)
        name = os.path.splitext(name)[0]
        self.image_names.append(name)

    
    def add_image(self, image_path):
        """添加单张图片"""
        try:
            original = pygame.image.load(image_path)  # 原始图片
            original_size = original.get_size()
            if original_size[0] > 500: original_size = (500, original_size[1])  # 限制最大宽度
            if original_size[1] > 500: original_size = (original_size[0], 500)  # 限制最大高度
            
            # 缩放到显示大小
            scaled = pygame.transform.scale(original, (self.thumbnail_size, self.thumbnail_size))
            
            self.original_images.append(original)  # 保存原始图片
            self.images.append(scaled)             # 保存缩放后的图片
            self.image_paths.append(image_path)
            self.original_images_size.append(original_size)
            
            # ===== 关键：同步添加图片名称 =====
            name = os.path.basename(image_path)
            name = os.path.splitext(name)[0]
            self.image_names.append(name)
            
            self._update_content_height()
            return True
        except Exception as e:
            print(f"加载失败 {image_path}: {e}")
            return False
    
    def add_images(self, image_paths):
        """批量添加图片"""
        for path in image_paths:
            self.add_image(path)

    def set_image_to_original_size(self, index):
        """缩放指定索引的图片为原来大小"""
        if 0 <= index < len(self.original_images):
            original = self.original_images[index]
            scaled = pygame.transform.scale(original, self.original_images_size[index])
            self.images[index] = scaled
            return True
        return False
    
    def add_test_images(self, count):
        """添加测试图片（演示用）"""
        colors = [(255,100,100), (100,255,100), (100,100,255), (255,255,100),
                  (255,100,255), (100,255,255), (255,200,100), (200,100,255),
                  (100,200,255), (255,150,150)]
        
        for i in range(count):
            surf = pygame.Surface((self.thumbnail_size, self.thumbnail_size))
            surf.fill(colors[i % len(colors)])
            
            font = pygame.font.Font(None, 36)
            text = font.render(str(i+1), True, WHITE)
            text_rect = text.get_rect(center=(self.thumbnail_size//2, self.thumbnail_size//2))
            surf.blit(text, text_rect)
            pygame.draw.rect(surf, BLACK, (0, 0, self.thumbnail_size, self.thumbnail_size), 2)
            
            self.images.append(surf)
            self.image_paths.append("test")
        
        self._update_content_height()
    
    def _update_content_height(self):
        """根据图片数量计算总高度"""
        if not self.images:
            self.total_content_height = 0
        else:
            # 计算需要多少行
            rows = math.ceil(len(self.images) / self.columns)
            self.total_content_height = rows * self.row_height + self.spacing
        
        self.scrollbar.set_content_size(self.total_content_height, self.height)
    
    def handle_event(self, event):
        """处理事件（滚轮、滚动条、鼠标拖动）"""
        # 鼠标滚轮
        if event.type == pygame.MOUSEWHEEL:
            mouse_x, mouse_y = pygame.mouse.get_pos()
            if self.x <= mouse_x <= self.x + self.width and self.y <= mouse_y <= self.y + self.height:
                self.scroll_offset -= event.y * 25
                self._clamp_scroll()
                self._sync_scrollbar()
        
        # 鼠标按下 - 开始拖动
        if event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
            mouse_x, mouse_y = event.pos
            self.drag_index = self.get_clicked_image_index(mouse_x, mouse_y)
            if self.drag_index != -1:
                self.dragging = True
                
                # 获取被点击图片的实际屏幕位置
                rect = self.get_image_rect(self.drag_index)
                
                # 计算鼠标相对于图片左上角的偏移
                self.drag_offset_x = mouse_x - rect.x
                self.drag_offset_y = mouse_y - rect.y
                
                self.drag_start_pos = (mouse_x, mouse_y)
                self.drag_start_image_pos = (rect.x, rect.y)
        
        # 鼠标移动 - 拖动中
        if event.type == pygame.MOUSEMOTION and self.dragging:
            mouse_x, mouse_y = event.pos
            # 计算图片应该出现的位置（鼠标位置减去偏移量）
            new_x = mouse_x - self.drag_offset_x
            new_y = mouse_y - self.drag_offset_y
            
            # 计算相对于原始位置的偏移
            dx = new_x - self.drag_start_image_pos[0]
            dy = new_y - self.drag_start_image_pos[1]
            
            self.drag_current_offset = (dx, dy)
        
        # 鼠标释放 - 结束拖动
        if event.type == pygame.MOUSEBUTTONUP and event.button == 1 and self.dragging:
            self.dragging = False
            self.drag_index = -1
            self.drag_current_offset = (0, 0)
        
        # 滚动条事件
        self.scrollbar.handle_event(event)
        self.scroll_offset = self.scrollbar.get_scroll_offset()
    
    def _clamp_scroll(self):
        """限制滚动范围"""
        max_scroll = max(0, self.total_content_height - self.height)
        self.scroll_offset = max(0, min(self.scroll_offset, max_scroll))
    
    def _sync_scrollbar(self):
        """同步滚动条位置"""
        if self.total_content_height <= self.height:
            return
        
        max_scroll = self.total_content_height - self.height
        scroll_ratio = self.scroll_offset / max_scroll if max_scroll > 0 else 0
        self.scrollbar.slider_y = self.scrollbar.rect.y + (scroll_ratio * self.scrollbar.max_slider_move)
        self.scrollbar.scroll_ratio = scroll_ratio
    
    def draw(self, screen, font_path):
        """绘制网格"""
        self.clip_surface.fill(self.bg_color)
        
        if not self.images:
            font = pygame.font.Font(font_path, 36)
            text = font.render("暂无图片", True, GRAY)
            text_rect = text.get_rect(center=(self.width//2, self.height//2))
            self.clip_surface.blit(text, text_rect)
        else:
            # 计算可见区域
            start_y = self.scroll_offset
            end_y = self.scroll_offset + self.height
            
            start_row = max(0, start_y // self.row_height)
            end_row = min(math.ceil(len(self.images) / self.columns), (end_y // self.row_height) + 2)
            
            # 绘制可见的图片
            for row in range(start_row, end_row):
                for col in range(self.columns):
                    idx = row * self.columns + col
                    if idx >= len(self.images):
                        break
                    
                    # 如果是正在拖动的图片，跳过（单独绘制）
                    if self.dragging and idx == self.drag_index:
                        continue
                    
                    # 计算位置
                    img_x = self.spacing + col * (self.thumbnail_size + self.spacing)
                    img_y = self.spacing + row * self.row_height - self.scroll_offset
                    
                    # 只绘制可见的
                    if img_y + self.thumbnail_size > 0 and img_y < self.height:
                        # 背景框
                        bg_rect = pygame.Rect(img_x - 3, img_y - 3, 
                                            self.thumbnail_size + 6, self.thumbnail_size + 6)
                        pygame.draw.rect(self.clip_surface, LIGHT_BLUE, bg_rect)
                        pygame.draw.rect(self.clip_surface, DARK_GRAY, bg_rect, 1)
                        
                        # 图片
                        self.clip_surface.blit(self.images[idx], (img_x, img_y))
                        
                        # ===== 显示图片名字 =====
                        name = self.image_names[idx] if idx < len(self.image_names) else f"图片{idx+1}"
                        font = pygame.font.Font(font_path, 14)
                        text = font.render(name, True, BLACK)
                        
                        # 文字位置（图片下方居中）
                        text_x = img_x + (self.thumbnail_size - text.get_width()) // 2
                        text_y = img_y + self.thumbnail_size + 2
                        
                        # 文字背景
                        bg_rect = pygame.Rect(text_x - 2, text_y - 1, text.get_width() + 4, text.get_height() + 2)
                        pygame.draw.rect(self.clip_surface, WHITE, bg_rect)
                        pygame.draw.rect(self.clip_surface, DARK_GRAY, bg_rect, 1)
                        
                        # 绘制文字
                        self.clip_surface.blit(text, (text_x, text_y))
        
        # 边框
        pygame.draw.rect(self.clip_surface, BLACK, (0, 0, self.width, self.height), 2)
        
        # 绘制网格区域到屏幕
        screen.blit(self.clip_surface, (self.x, self.y))
        
        # 滚动条
        self.scrollbar.draw(screen)
        
        # 绘制被拖动的图片（如果正在拖动）
        if self.dragging and self.drag_index != -1:
            # ===== 最后绘制被拖动的图片（真正的最上层）=====
            idx = self.drag_index
            # 计算原始位置
            row = idx // self.columns
            col = idx % self.columns
            original_x = self.spacing + col * (self.thumbnail_size + self.spacing)
            original_y = self.spacing + row * self.row_height - self.scroll_offset
            
            # 加上偏移量，并转换到屏幕坐标
            img_x = self.x + original_x + self.drag_current_offset[0]
            img_y = self.y + original_y + self.drag_current_offset[1]
            
            # 绘制阴影
            shadow_rect = pygame.Rect(img_x + 5, img_y + 5, self.thumbnail_size, self.thumbnail_size)
            pygame.draw.rect(screen, (100, 100, 100), shadow_rect)
            
            # 绘制图片
            screen.blit(self.images[idx], (img_x, img_y))
            
            # 绘制高亮边框
            pygame.draw.rect(screen, RED, (img_x, img_y, self.thumbnail_size, self.thumbnail_size), 3)
            
            # ===== 拖动时也显示图片名字 =====
            name = self.image_names[idx] if idx < len(self.image_names) else f"图片{idx+1}"
            font = pygame.font.Font(font_path, 14)
            text = font.render(name, True, BLACK)
            
            # 文字位置（图片下方居中）
            text_x = img_x + (self.thumbnail_size - text.get_width()) // 2
            text_y = img_y + self.thumbnail_size + 2
            
            # 文字背景
            bg_rect = pygame.Rect(text_x - 2, text_y - 1, text.get_width() + 4, text.get_height() + 2)
            pygame.draw.rect(screen, WHITE, bg_rect)
            pygame.draw.rect(screen, DARK_GRAY, bg_rect, 1)
            
            # 绘制文字
            screen.blit(text, (text_x, text_y))


    def get_clicked_image_index(self, mouse_x, mouse_y):
        """根据鼠标坐标返回点击的图片索引，如果没有点击到返回-1"""
        # 先检查鼠标是否在网格可视区域内
        if not (self.x <= mouse_x <= self.x + self.width and self.y <= mouse_y <= self.y + self.height):
            return -1
        
        # 计算相对于网格区域的坐标（注意：这里不加 scroll_offset）
        local_x = mouse_x - self.x
        local_y = mouse_y - self.y  # 不加滚动偏移
        
        # 获取当前可视区域顶部对应的内容位置
        view_top = self.scroll_offset
        
        # 计算鼠标在内容中的实际位置
        content_y = view_top + local_y
        
        # 完整的行高
        full_row_height = self.thumbnail_size + self.spacing + self.name_height
        
        # 计算行号
        row = content_y // full_row_height
        col = local_x // (self.thumbnail_size + self.spacing)
        
        # 边界检查
        if col >= self.columns or row < 0 or row >= len(self.images) // self.columns + 1:
            return -1
        
        # 计算索引
        index = row * self.columns + col
        
        # 验证索引有效
        if 0 <= index < len(self.images):
            # 计算该图片在内容中的实际位置
            img_row = index // self.columns
            img_col = index % self.columns
            img_content_y = img_row * full_row_height + self.spacing
            img_content_x = img_col * (self.thumbnail_size + self.spacing) + self.spacing
            
            # 检查鼠标是否在图片区域内
            img_top = img_content_y - view_top  # 屏幕上的Y坐标
            img_left = img_content_x
            
            if (img_left <= local_x <= img_left + self.thumbnail_size and 
                img_top <= local_y <= img_top + self.thumbnail_size):
                return index
        
        return -1

    def get_image_rect(self, index):
        """获取图片在屏幕上的矩形区域"""
        row = index // self.columns
        col = index % self.columns
        
        img_x = self.x + self.spacing + col * (self.thumbnail_size + self.spacing)
        img_y = self.y + self.spacing + row * self.row_height - self.scroll_offset
        
        return pygame.Rect(img_x, img_y, self.thumbnail_size, self.thumbnail_size)
    
    def get_drag_image_rect(self, index):
        """获取图片在屏幕上的矩形区域（自动处理拖动时的偏移）"""
        row = index // self.columns
        col = index % self.columns
        
        img_x = self.x + self.spacing + col * (self.thumbnail_size + self.spacing)
        img_y = self.y + self.spacing + row * self.row_height - self.scroll_offset
        
        # 如果是正在被拖动的图片，加上偏移量
        if self.dragging and index == self.drag_index:
            img_x += self.drag_current_offset[0]
            img_y += self.drag_current_offset[1]
        
        return pygame.Rect(img_x, img_y, self.thumbnail_size, self.thumbnail_size)

    def get_image_path(self, index):
        """获取指定索引的图片路径"""
        if 0 <= index < len(self.image_paths):
            return self.image_paths[index]
        return None



    def set_image_name(self, index, new_name):
        """修改指定索引的图片名字"""
        if 0 <= index < len(self.image_names):
            self.image_names[index] = new_name
            return True
        return False

    def get_image_name(self, index):
        """获取指定索引的图片名字"""
        if 0 <= index < len(self.image_names):
            return self.image_names[index]
        return None


    def get_hover_image_index(self, mouse_x, mouse_y):
        """获取鼠标悬停的图片索引，如果没有返回-1"""
        # 先检查鼠标是否在网格区域内
        if not (self.x <= mouse_x <= self.x + self.width and self.y <= mouse_y <= self.y + self.height):
            return -1
        
        # 计算相对于网格区域的坐标（不加滚动偏移）
        local_x = mouse_x - self.x
        local_y = mouse_y - self.y
        
        # 获取当前可视区域顶部对应的内容位置
        view_top = self.scroll_offset
        
        # 计算鼠标在内容中的实际位置
        content_y = view_top + local_y
        
        # 完整的行高
        full_row_height = self.thumbnail_size + self.spacing + self.name_height
        
        # 计算行号和列号
        col = int(local_x // (self.thumbnail_size + self.spacing))
        row = int(content_y // full_row_height)
        
        # 边界检查
        if col >= self.columns or row < 0:
            return -1
        
        # 计算图片索引
        index = row * self.columns + col
        
        # 检查索引是否有效
        if 0 <= index < len(self.images):
            # 计算该图片在内容中的实际位置
            img_row = index // self.columns
            img_col = index % self.columns
            img_content_y = img_row * full_row_height + self.spacing
            img_content_x = img_col * (self.thumbnail_size + self.spacing) + self.spacing
            
            # 检查鼠标是否在图片区域内
            img_top = img_content_y - view_top
            img_left = img_content_x
            
            if (img_left <= local_x <= img_left + self.thumbnail_size and 
                img_top <= local_y <= img_top + self.thumbnail_size):
                return index
        
        return -1

    def quit():
        """退出"""
        pygame.quit()
        sys.exit()



if __name__ == "__main__":
    # 设置：显示区域在右下角
    # 宽度300，高度窗口的一半多一点（500）
    grid_width = 320  # 宽度（包含间距）
    grid_height = 500  # 高度
    
    # 位置：右下角，留出边距
    margin = 20
    x = WINDOW_WIDTH - grid_width - margin
    y = WINDOW_HEIGHT - grid_height - margin
    
    # 创建网格（2列显示）
    grid = InfiniteImageGrid(
        x=x, 
        y=y, 
        width=grid_width, 
        height=grid_height,
        columns=2,          # 2列
        thumbnail_size=130, # 缩略图大小
        spacing=12,         # 间距
        bg_color=(240, 240, 240)
    )

    font_path = os.path.join(os.path.dirname(__file__), "font/MSYHBD.TTC")
    
    # 添加图片的几种方式：
    
    # 方式1：自动生成测试图片（演示无限滚动）
    print("生成50张测试图片演示无限滚动...")
    grid.add_test_images(50)  # 可以改成100、200等任意数量
    
    # 方式2：添加真实图片（取消注释）
    # import glob
    # image_files = glob.glob("*.jpg") + glob.glob("*.png")
    # grid.add_images(image_files)
    
    # 方式3：手动添加单张
    # grid.add_image("photo1.jpg")
    
    running = True
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            
            # 传递给网格处理
            grid.handle_event(event)
        
        # 绘制
        screen.fill(WHITE)
        
        # 绘制标题和说明
        font_title = pygame.font.Font(font_path, 36)
        title = font_title.render("图片网格 - 2列显示 | 无限滚动", True, BLACK)
        screen.blit(title, (margin, margin))
        
        font_info = pygame.font.Font(font_path, 24)
        info1 = font_info.render(f"共 {len(grid.images)} 张图片", True, DARK_GRAY)
        info2 = font_info.render("操作：拖拽右侧滚动条 或 鼠标滚轮", True, DARK_GRAY)
        info3 = font_info.render(f"当前滚动: {grid.scroll_offset} / {max(0, grid.total_content_height - grid.height)}", True, DARK_GRAY)
        
        screen.blit(info1, (margin, margin + 50))
        screen.blit(info2, (margin, margin + 80))
        screen.blit(info3, (margin, margin + 110))
        
        # 绘制网格区域说明
        pygame.draw.rect(screen, (200, 200, 200), (x-5, y-5, grid_width+10, grid_height+10), 1)
        label = font_info.render("图片显示区域", True, GRAY)
        screen.blit(label, (x+10, y-25))
        
        # 绘制网格
        grid.draw(screen, font_path)
        
        pygame.display.flip()
        clock.tick(60)
    
    pygame.quit()
    sys.exit()



import pygame
import sys
import math
import os
from typing import Tuple


WINDOW_WIDTH: int
WINDOW_HEIGHT: int
screen: pygame.Surface
clock: pygame.time.Clock


WHITE: Tuple[int, int, int]
BLACK: Tuple[int, int, int]
GRAY: Tuple[int, int, int]
DARK_GRAY: Tuple[int, int, int]
BLUE: Tuple[int, int, int]
RED: Tuple[int, int, int]
LIGHT_BLUE: Tuple[int, int, int]

class ScrollBar:
    """
    Vertical scroll bar


    垂直滚动条
    """
    def __init__(self, x, y, height, content_height, view_height) -> None:
        ...

    def update_slider(self) -> None:
        """
        Update slider size and position


        更新滑块大小和位置
        """

    def set_content_size(self, content_height, view_height) -> None:
        ...

    def handle_event(self, event) -> None:
        ...

    def get_scroll_offset(self) -> int:
        ...

    def draw(self, screen) -> None:
        ...


class InfiniteImageGrid:
    """
    Infinite scrolling image grid (multi-column layout)


    无限滚动的图片网格（多列布局）
    """
    def __init__(self, x, y, width, height, columns=2, thumbnail_size=120, spacing=15, name_height=20, bg_color=(30, 30, 30)) -> None:
        ...

    def add_image_name(self, image_path) -> None:
        """
        Set the name of the specified image


        设置指定图片的名字
        """

    def add_image(self, image_path) -> None:
        """
        Add a single image


        添加单张图片
        """

    def add_images(self, image_paths) -> None:
        """
        Add multiple images in batch


        批量添加图片
        """

    def set_image_to_original_size(self, index) -> None:
        """
        Scale the specified image to its original size


        缩放指定索引的图片为原来大小
        """

    def add_test_images(self, count) -> None:
        """
        Add test images (for demonstration)


        添加测试图片（演示用）
        """

    def handle_event(self, event) -> None:
        """
        Handle events (scroll wheel, scroll bar, mouse drag)


        处理事件（滚轮、滚动条、鼠标拖动）
        """

    def draw(self, screen, font_path) -> None:
        """
        Draw the grid


        绘制网格
        """

    def get_clicked_image_index(self, mouse_x, mouse_y) -> int:
        """
        Return the index of the clicked image based on mouse coordinates, return -1 if none clicked


        根据鼠标坐标返回点击的图片索引，如果没有点击到返回-1
        """

    def get_image_rect(self, index) -> pygame.Rect:
        """
        Get the rectangular area of the image on the screen


        获取图片在屏幕上的矩形区域
        """

    def get_drag_image_rect(self, index) -> pygame.Rect:
        """
        Get the rectangular area of the image on the screen (automatically handles drag offset)


        获取图片在屏幕上的矩形区域（自动处理拖动时的偏移）
        """

    def get_image_path(self, index) -> str:
        """
        Get the path of the specified image


        获取指定索引的图片路径
        """

    def set_image_name(self, index, new_name) -> bool:
        """
        Modify the name of the specified image


        修改指定索引的图片名字
        """

    def get_image_name(self, index) -> str:
        """
        Get the name of the specified image


        获取指定索引的图片名字
        """

    def get_hover_image_index(self, mouse_x, mouse_y) -> int:
        """
        Get the index of the image the mouse is hovering over, return -1 if none


        获取鼠标悬停的图片索引，如果没有返回-1
        """

    def quit() -> None:
        """
        Quit


        退出
        """
        pygame.quit()
        sys.exit()


from setuptools import setup

setup(
    name="2D-Animation-lib",
    version="1.4.4",
    packages=["Animation"], 
    package_data={
        "":["LICENSE"], 
        "Animation":[
            "py.typed", 
            "*.py", 
            "*.pyi", 
            "examples/*.png", 
            "method/*.py", "method/*.pyi", 
            "font/*.TTC"
        ]
    }, 
    include_package_data=True, 
    install_requires=["pygame>=2.0.0"],
    author="WatermelonCode",
    description="A 2D simple animation module for pygame.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    license="MIT", 
    entry_points={
        "console_scripts":[
            "run-Animation-example=Animation.__main__:main", 
            "run-Animation-example-camera=Animation.__main__:camera_demo"
        ]
    }
)


# ========== 前景色（字体颜色） ==========
def black(text): return f"\033[30m{text}\033[0m"      # 黑色
def red(text): return f"\033[31m{text}\033[0m"        # 红色
def green(text): return f"\033[32m{text}\033[0m"      # 绿色
def yellow(text): return f"\033[33m{text}\033[0m"     # 黄色
def blue(text): return f"\033[34m{text}\033[0m"       # 蓝色
def magenta(text): return f"\033[35m{text}\033[0m"    # 品红/洋红
def cyan(text): return f"\033[36m{text}\033[0m"       # 青色
def white(text): return f"\033[37m{text}\033[0m"      # 白色

# 亮色 / 高亮前景 (90–97)
def bright_black(text): return f"\033[90m{text}\033[0m"   # 亮黑（灰）
def bright_red(text): return f"\033[91m{text}\033[0m"     # 亮红
def bright_green(text): return f"\033[92m{text}\033[0m"   # 亮绿
def bright_yellow(text): return f"\033[93m{text}\033[0m"  # 亮黄
def bright_blue(text): return f"\033[94m{text}\033[0m"    # 亮蓝
def bright_magenta(text): return f"\033[95m{text}\033[0m" # 亮品红
def bright_cyan(text): return f"\033[96m{text}\033[0m"    # 亮青
def bright_white(text): return f"\033[97m{text}\033[0m"   # 亮白

# ========== 背景色 ==========
def bg_black(text): return f"\033[40m{text}\033[0m"
def bg_red(text): return f"\033[41m{text}\033[0m"
def bg_green(text): return f"\033[42m{text}\033[0m"
def bg_yellow(text): return f"\033[43m{text}\033[0m"
def bg_blue(text): return f"\033[44m{text}\033[0m"
def bg_magenta(text): return f"\033[45m{text}\033[0m"
def bg_cyan(text): return f"\033[46m{text}\033[0m"
def bg_white(text): return f"\033[47m{text}\033[0m"

# 亮背景 (100–107)
def bg_bright_black(text): return f"\033[100m{text}\033[0m"
def bg_bright_red(text): return f"\033[101m{text}\033[0m"
def bg_bright_green(text): return f"\033[102m{text}\033[0m"
def bg_bright_yellow(text): return f"\033[103m{text}\033[0m"
def bg_bright_blue(text): return f"\033[104m{text}\033[0m"
def bg_bright_magenta(text): return f"\033[105m{text}\033[0m"
def bg_bright_cyan(text): return f"\033[106m{text}\033[0m"
def bg_bright_white(text): return f"\033[107m{text}\033[0m"

# ========== 样式 ==========
def bold(text): return f"\033[1m{text}\033[0m"       # 粗体
def dim(text): return f"\033[2m{text}\033[0m"        # 暗淡
def italic(text): return f"\033[3m{text}\033[0m"     # 斜体（部分终端支持）
def underline(text): return f"\033[4m{text}\033[0m"  # 下划线
def blink(text): return f"\033[5m{text}\033[0m"      # 闪烁（少用）
def reverse(text): return f"\033[7m{text}\033[0m"    # 反色（背景↔前景）
def hidden(text): return f"\033[8m{text}\033[0m"     # 隐藏（慎用）

# ========== 常用别名 ==========
def gray(text): return bright_black(text)   # 灰色就是亮黑
def purple(text): return magenta(text)      # 紫色常用品红替代
def pink(text): return bright_magenta(text) # 粉红 ≈ 亮品红
def orange(text): return f"\033[38;2;255;165;0m{text}\033[0m"  # 橙色


def rgb2hex(r, g, b): return f"#{r:02X}{g:02X}{b:02X}"
def hex2rgb(hex_str):
    hex_str = hex_str.lstrip('#')
    r, g, b = tuple(int(hex_str[i:i+2], 16) for i in (0, 2, 4))
    return r, g, b


import math

class Line:
    def __init__(self, x1, y1, x2, y2, thickness=5):
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2
        self.thickness = thickness
        self._half_thickness = thickness / 2
    
    def update(self, x1=None, y1=None, x2=None, y2=None):
        if x1 is not None:
            self.x1 = x1
        if y1 is not None:
            self.y1 = y1
        if x2 is not None:
            self.x2 = x2
        if y2 is not None:
            self.y2 = y2
    
    def set_thickness(self, thickness):
        self.thickness = thickness
        self._half_thickness = thickness / 2
    
    def get_start_point(self):
        return (self.x1, self.y1)
    
    def get_end_point(self):
        return (self.x2, self.y2)
    
    def get_center_point(self):
        return ((self.x1 + self.x2) / 2, (self.y1 + self.y2) / 2)
    
    def get_length(self):
        return math.hypot(self.x2 - self.x1, self.y2 - self.y1)
    
    def get_direction(self):
        dx = self.x2 - self.x1
        dy = self.y2 - self.y1
        length = self.get_length()
        if length == 0:
            return (0, 0)
        return (dx / length, dy / length)
    
    def get_normal(self):
        dx, dy = self.get_direction()
        return (-dy, dx)  # 顺时针旋转90度
    
    def _point_to_segment_distance(self, px, py):
        dx = self.x2 - self.x1
        dy = self.y2 - self.y1
        
        # 如果线段是一个点
        if dx == 0 and dy == 0:
            return math.hypot(px - self.x1, py - self.y1)
        
        # 计算投影参数 t
        t = ((px - self.x1) * dx + (py - self.y1) * dy) / (dx * dx + dy * dy)
        
        # 限制 t 在 [0, 1] 范围内
        t = max(0, min(1, t))
        
        # 计算最近点的坐标
        closest_x = self.x1 + t * dx
        closest_y = self.y1 + t * dy
        
        # 计算距离
        return math.hypot(px - closest_x, py - closest_y)
    
    def collisions_with_point(self, px, py, extra_tolerance=0):
        distance = self._point_to_segment_distance(px, py)
        return distance <= self._half_thickness + extra_tolerance
    
    def collisions_with_line(self, other_line):
        half_t1 = self._half_thickness
        half_t2 = other_line._half_thickness
        total_half = half_t1 + half_t2
        
        # 方法1：检查端点圆是否与另一条线段相交
        if other_line.collisions_with_point(self.x1, self.y1, total_half):
            return True
        if other_line.collisions_with_point(self.x2, self.y2, total_half):
            return True
        if self.collisions_with_point(other_line.x1, other_line.y1, total_half):
            return True
        if self.collisions_with_point(other_line.x2, other_line.y2, total_half):
            return True
        
        # 方法2：检查两条无限细的线段是否相交
        if self._line_line_collision(other_line):
            return True
        
        # 方法3：检查两条线段是否平行且距离很近
        min_distance = min(
            other_line._point_to_segment_distance(self.x1, self.y1),
            other_line._point_to_segment_distance(self.x2, self.y2),
            self._point_to_segment_distance(other_line.x1, other_line.y1),
            self._point_to_segment_distance(other_line.x2, other_line.y2)
        )
        
        return min_distance <= total_half
    
    def _line_line_collision(self, other_line):
        def ccw(ax, ay, bx, by, cx, cy):
            return (cy - ay) * (bx - ax) > (by - ay) * (cx - ax)
        
        return (ccw(self.x1, self.y1, other_line.x1, other_line.y1, other_line.x2, other_line.y2) != 
                ccw(self.x2, self.y2, other_line.x1, other_line.y1, other_line.x2, other_line.y2) and 
                ccw(self.x1, self.y1, self.x2, self.y2, other_line.x1, other_line.y1) != 
                ccw(self.x1, self.y1, self.x2, self.y2, other_line.x2, other_line.y2))
    
    def collisions_with_rect(self, rect_x, rect_y, rect_w, rect_h):
        # 1. 检查矩形的四个角点是否在线段的厚度范围内
        corners = [
            (rect_x, rect_y),
            (rect_x + rect_w, rect_y),
            (rect_x, rect_y + rect_h),
            (rect_x + rect_w, rect_y + rect_h)
        ]
        
        for corner_x, corner_y in corners:
            if self.collisions_with_point(corner_x, corner_y):
                return True
        
        # 2. 检查矩形的四条边是否与线段相交
        edges = [
            Line(rect_x, rect_y, rect_x + rect_w, rect_y, thickness=0),  # 上边
            Line(rect_x, rect_y + rect_h, rect_x + rect_w, rect_y + rect_h, thickness=0),  # 下边
            Line(rect_x, rect_y, rect_x, rect_y + rect_h, thickness=0),  # 左边
            Line(rect_x + rect_w, rect_y, rect_x + rect_w, rect_y + rect_h, thickness=0)  # 右边
        ]
        
        for edge in edges:
            if self.collisions_with_line(edge):
                return True
        
        # 3. 检查线段的端点圆是否与矩形相交
        if self._circle_rect_collision(self.x1, self.y1, self._half_thickness, rect_x, rect_y, rect_w, rect_h):
            return True
        if self._circle_rect_collision(self.x2, self.y2, self._half_thickness, rect_x, rect_y, rect_w, rect_h):
            return True
        
        return False
    
    @staticmethod
    def _circle_rect_collision(circle_x, circle_y, radius, rect_x, rect_y, rect_w, rect_h):
        closest_x = max(rect_x, min(circle_x, rect_x + rect_w))
        closest_y = max(rect_y, min(circle_y, rect_y + rect_h))
        distance = math.hypot(circle_x - closest_x, circle_y - closest_y)
        return distance <= radius
    
    def collisions_with_circle(self, circle_x, circle_y, radius):
        total_radius = self._half_thickness + radius
        
        # 检查端点圆是否与目标圆相交
        if math.hypot(self.x1 - circle_x, self.y1 - circle_y) <= total_radius:
            return True
        if math.hypot(self.x2 - circle_x, self.y2 - circle_y) <= total_radius:
            return True
        
        # 检查线段到圆心的距离
        distance = self._point_to_segment_distance(circle_x, circle_y)
        return distance <= total_radius
    
    def get_bounding_rect(self):
        min_x = min(self.x1, self.x2) - self._half_thickness
        min_y = min(self.y1, self.y2) - self._half_thickness
        max_x = max(self.x1, self.x2) + self._half_thickness
        max_y = max(self.y1, self.y2) + self._half_thickness
        
        return (min_x, min_y, max_x - min_x, max_y - min_y)
    
    def draw(self, screen, color, width=None):
        import pygame
        draw_width = width if width is not None else self.thickness
        pygame.draw.line(screen, color, (self.x1, self.y1), (self.x2, self.y2), draw_width)
    
    def draw_control_points(self, screen, color, radius=8):
        import pygame
        pygame.draw.circle(screen, color, (int(self.x1), int(self.y1)), radius)
        pygame.draw.circle(screen, color, (int(self.x2), int(self.y2)), radius)
    
    def __repr__(self):
        return f"Line(({self.x1}, {self.y1}) -> ({self.x2}, {self.y2}), thickness={self.thickness})"
    